@@ -1,9 +1,9 @@
 import colorsys
+import math
 import os
 from csv import writer as csv_writer
-from functools import reduce
 from pathlib import Path
-from typing import Dict, Iterable, List, Literal, Optional, Set, Tuple, get_args
+from typing import Dict, Iterable, List, Optional, Set, Tuple, get_args
 
 import numpy as np
 
@@ -16,7 +16,7 @@ from upolygon import draw_polygon
 
 import darwin.datatypes as dt
 from darwin.exceptions import DarwinException
-from darwin.utils import convert_polygons_to_sequences, ispolygon
+from darwin.utils import convert_polygons_to_sequences
 
 
 def get_palette(mode: dt.MaskTypes.Mode, categories: List[str]) -> dt.MaskTypes.Palette:
@@ -36,7 +36,7 @@ def get_palette(mode: dt.MaskTypes.Mode, categories: List[str]) -> dt.MaskTypes.
         A dict of categories and their corresponding palette value.
     """
 
-    if not mode in get_args(dt.MaskTypes.Mode):
+    if mode not in get_args(dt.MaskTypes.Mode):
         raise ValueError(f"Unknown mode {mode}.") from DarwinException
 
     if not isinstance(categories, list) or not categories:
@@ -52,9 +52,13 @@ def get_palette(mode: dt.MaskTypes.Mode, categories: List[str]) -> dt.MaskTypes.
         if num_categories > 254:
             raise ValueError("maximum number of classes supported: 254.")
         elif num_categories == 1:
-            raise ValueError("only having the '__background__' class is not allowed. Please add more classes.")
+            raise ValueError(
+                "only having the '__background__' class is not allowed. Please add more classes."
+            )
 
-        palette = {c: int(i * 255 / (num_categories - 1)) for i, c in enumerate(categories)}
+        palette = {
+            c: int(i * 255 / (num_categories - 1)) for i, c in enumerate(categories)
+        }
 
     if mode == "rgb":
         if num_categories > 360:
@@ -62,12 +66,16 @@ def get_palette(mode: dt.MaskTypes.Mode, categories: List[str]) -> dt.MaskTypes.
         palette = {c: i for i, c in enumerate(categories)}
 
     if not palette:
-        raise ValueError(f"Failed to generate a palette.", mode, categories) from DarwinException
+        raise ValueError(
+            "Failed to generate a palette.", mode, categories
+        ) from DarwinException
 
     return palette
 
 
-def get_rgb_colours(categories: dt.MaskTypes.CategoryList) -> Tuple[dt.MaskTypes.RgbColors, dt.MaskTypes.RgbPalette]:
+def get_rgb_colours(
+    categories: dt.MaskTypes.CategoryList,
+) -> Tuple[dt.MaskTypes.RgbColors, dt.MaskTypes.RgbPalette]:
     """
     Returns a list of RGB colours and a dict of categories and their corresponding RGB palette value.
 
@@ -89,14 +97,15 @@ def get_rgb_colours(categories: dt.MaskTypes.CategoryList) -> Tuple[dt.MaskTypes
     SATURATION_OF_COLOUR: float = 0.8
     VALUE_OF_COLOUR: float = 1.0
     hsv_colours: dt.MaskTypes.HsvColors = [
-        (x / num_categories, SATURATION_OF_COLOUR, VALUE_OF_COLOUR) for x in range(num_categories - 1)
+        (x / num_categories, SATURATION_OF_COLOUR, VALUE_OF_COLOUR)
+        for x in range(num_categories - 1)
+    ]
+    rgb_colour_list: dt.MaskTypes.RgbColorList = [
+        [int(e * 255) for e in colorsys.hsv_to_rgb(*x)] for x in hsv_colours
     ]
-    rgb_colour_list: dt.MaskTypes.RgbColorList = list(
-        map(lambda x: [int(e * 255) for e in colorsys.hsv_to_rgb(*x)], hsv_colours)
-    )
     # Now we add BG class with [0 0 0] RGB value
     rgb_colour_list.insert(0, [0, 0, 0])
-    palette_rgb: dt.MaskTypes.RgbPalette = {c: rgb for c, rgb in zip(categories, rgb_colour_list)}
+    palette_rgb: dt.MaskTypes.RgbPalette = dict(zip(categories, rgb_colour_list))
     rgb_colours: dt.MaskTypes.RgbColors = [c for e in rgb_colour_list for c in e]
 
     return rgb_colours, palette_rgb
@@ -116,21 +125,27 @@ def get_render_mode(annotations: List[dt.AnnotationLike]) -> dt.MaskTypes.TypeOf
     TypeOfRenderType
         A string reading either "raster" or "polygon".
     """
-    non_video_annotations: List[dt.Annotation] = [a for a in annotations if not isinstance(a, dt.VideoAnnotation)]
+    non_video_annotations: List[dt.Annotation] = [
+        a for a in annotations if not isinstance(a, dt.VideoAnnotation)
+    ]
 
     if not non_video_annotations:
         return "polygon"
 
-    list_of_keys: List[str] = reduce(list.__add__, [list(a.data.keys()) for a in non_video_annotations])
-    keys: Set[str] = set(list_of_keys)
+    list_of_types: List[str] = [
+        a.annotation_class.annotation_type for a in non_video_annotations
+    ]
+    types: Set[str] = set(list_of_types)
 
-    is_raster_mask = ("mask" in keys) and ("raster_layer" in keys)
-    is_polygon = ("path" in keys) or ("paths" in keys)
+    is_raster_mask = ("mask" in types) and ("raster_layer" in types)
+    is_polygon = ("polygon" in types) or ("complex_polygon" in types)
 
-    raster_layer_count = len([a for a in keys if a == "raster_layer"])
+    raster_layer_count = len([a for a in types if a == "raster_layer"])
 
     if is_raster_mask and is_polygon:
-        raise ValueError("Cannot have both raster and polygon annotations in the same file")
+        raise ValueError(
+            "Cannot have both raster and polygon annotations in the same file"
+        )
 
     if is_raster_mask and raster_layer_count > 1:
         raise ValueError("Cannot have more than one raster layer in the same file")
@@ -141,7 +156,10 @@ def get_render_mode(annotations: List[dt.AnnotationLike]) -> dt.MaskTypes.TypeOf
     if is_polygon:
         return "polygon"
 
-    raise ValueError("No renderable annotations found in file, found keys: " + ",".join(keys))
+    raise ValueError(
+        "No renderable annotations found in file, found types: "
+        + ",".join(list_of_types)
+    )
 
 
 def colours_in_rle(
@@ -170,9 +188,11 @@ def colours_in_rle(
         mask: Optional[dt.AnnotationMask] = mask_lookup.get(uuid)
 
         if mask is None:
-            raise ValueError(f"Could not find mask with uuid {uuid} in mask lookup table.")
+            raise ValueError(
+                f"Could not find mask with uuid {uuid} in mask lookup table."
+            )
 
-        if not mask.name in colours:
+        if mask.name not in colours:
             colours[mask.name] = colour_value
 
     return colours  # Returns same item as the outset, technically not needed, but best practice.
@@ -212,7 +232,7 @@ def get_or_generate_colour(cat_name: str, colours: dt.MaskTypes.ColoursDict) ->
     -------
     int - the integer for the colour name.  These will later be reassigned to a wider spread across the colour spectrum.
     """
-    if not cat_name in colours:
+    if cat_name not in colours:
         colours[cat_name] = len(colours) + 1
 
     return colours[cat_name]
@@ -254,14 +274,23 @@ def render_polygons(
 
     errors: List[Exception] = []
 
-    for a in annotations:
-        try:
-            if isinstance(a, dt.VideoAnnotation):
-                print(f"Skipping video annotation from file {annotation_file.filename}")
-                continue
+    filtered_annotations: List[dt.Annotation] = [
+        a for a in annotations if not isinstance(a, dt.VideoAnnotation)
+    ]
+    beyond_window = annotations_exceed_window(filtered_annotations, height, width)
+    if beyond_window:
+        # If the annotations exceed the window, we need to offset the mask to fit them all in.
+        # Capture the offsets so we can shift the annotations back to their original positions later
+        x_min, x_max, y_min, y_max = get_extents(filtered_annotations, height, width)
+        new_height = y_max - y_min
+        new_width = x_max - x_min
+        mask = np.zeros((new_height, new_width), dtype=np.uint8)
+        offset_x, offset_y = -x_min, -y_min
 
+    for a in filtered_annotations:
+        try:
             cat = a.annotation_class.name
-            if not cat in categories:
+            if cat not in categories:
                 categories.append(cat)
 
             if a.annotation_class.annotation_type == "polygon":
@@ -269,10 +298,21 @@ def render_polygons(
             elif a.annotation_class.annotation_type == "complex_polygon":
                 polygon = a.data["paths"]
             else:
-                raise ValueError(f"Unknown annotation type {a.annotation_class.annotation_type}")
-            sequence = convert_polygons_to_sequences(polygon, height=height, width=width)
+                raise ValueError(
+                    f"Unknown annotation type {a.annotation_class.annotation_type}"
+                )
 
-            colour_to_draw = get_or_generate_colour(cat, colours)
+            if beyond_window:
+                # Offset the polygon by the minimum x and y values to shift it to new frame of reference
+                polygon_off = offset_polygon(polygon, offset_x, offset_y)
+                sequence = convert_polygons_to_sequences(
+                    polygon_off, height=new_height, width=new_width
+                )
+            else:
+                sequence = convert_polygons_to_sequences(
+                    polygon, height=height, width=width
+                )
+            colour_to_draw = categories.index(cat)
             mask = draw_polygon(mask, sequence, colour_to_draw)
 
             if cat not in colours:
@@ -282,6 +322,9 @@ def render_polygons(
             errors.append(e)
             continue
 
+    if beyond_window:
+        # crop the mask to the original image size and in the correct offset location
+        mask = mask[offset_y : offset_y + height, offset_x : offset_x + width]
     # It's not necessary to return the mask, it's modified in place, but it's more explicit
     return errors, mask, categories, colours
 
@@ -322,7 +365,7 @@ def render_raster(
     mask_annotations: List[dt.AnnotationMask] = []
     raster_layer: Optional[dt.RasterLayer] = None
 
-    mask_lookup: Dict[str, dt.AnnotationMask] = dict()
+    mask_lookup: Dict[str, dt.AnnotationMask] = {}
 
     for a in annotations:
         if isinstance(a, dt.VideoAnnotation):
@@ -330,10 +373,10 @@ def render_raster(
 
         data = a.data
 
-        if "mask" in data:
+        if a.annotation_class.annotation_type == "mask" and a.id:
             new_mask = dt.AnnotationMask(
-                id=data["id"],
-                name=data["name"],
+                id=a.id,
+                name=a.annotation_class.name,
                 slot_names=a.slot_names,
             )
             try:
@@ -344,16 +387,18 @@ def render_raster(
 
             mask_annotations.append(new_mask)
 
-            if not new_mask.id in mask_lookup:
+            if new_mask.id not in mask_lookup:
                 mask_lookup[new_mask.id] = new_mask
 
             # Add the category to the list of categories
-            if not new_mask.name in categories:
+            if new_mask.name not in categories:
                 categories.append(new_mask.name)
 
-        if "raster_layer" in data and (rl := data["raster_layer"]):
+        if a.annotation_class.annotation_type == "raster_layer" and (rl := data):
             if raster_layer:
-                errors.append(ValueError(f"Annotation {a.id} has more than one raster layer"))
+                errors.append(
+                    ValueError(f"Annotation {a.id} has more than one raster layer")
+                )
                 break
 
             new_rl = dt.RasterLayer(
@@ -367,11 +412,11 @@ def render_raster(
             raster_layer = new_rl
 
     if not raster_layer:
-        errors.append(ValueError(f"Annotation has no raster layer"))
+        errors.append(ValueError("Annotation has no raster layer"))
         return errors, mask, categories, colours
 
     if not mask_annotations:
-        errors.append(ValueError(f"Annotation has no masks"))
+        errors.append(ValueError("Annotation has no masks"))
         return errors, mask, categories, colours
 
     try:
@@ -384,21 +429,30 @@ def render_raster(
     return errors, mask, categories, colours
 
 
-def export(annotation_files: Iterable[dt.AnnotationFile], output_dir: Path, mode: dt.MaskTypes.Mode) -> None:
+def export(
+    annotation_files: Iterable[dt.AnnotationFile],
+    output_dir: Path,
+    mode: dt.MaskTypes.Mode,
+) -> None:
     masks_dir: Path = output_dir / "masks"
     masks_dir.mkdir(exist_ok=True, parents=True)
     annotation_files = list(annotation_files)
-
-    all_classes_sets: List[Set[dt.AnnotationClass]] = [a.annotation_classes for a in annotation_files]
+    accepted_types = ["polygon", "complex_polygon", "raster_layer", "mask"]
+    all_classes_sets: List[Set[dt.AnnotationClass]] = [
+        a.annotation_classes for a in annotation_files
+    ]
     if len(all_classes_sets) > 0:
         all_classes: Set[dt.AnnotationClass] = set.union(*all_classes_sets)
-        categories: List[str] = ["__background__"] + [c.name for c in list(all_classes)]
+        categories: List[str] = ["__background__"] + sorted(
+            {c.name for c in all_classes if c.annotation_type in accepted_types},
+            key=lambda x: x.lower(),
+        )
         palette = get_palette(mode, categories)
     else:
         categories = ["__background__"]
-        palette = dict()
+        palette = {}
 
-    colours: dt.MaskTypes.ColoursDict = dict()
+    colours: dt.MaskTypes.ColoursDict = {}
 
     for annotation_file in annotation_files:
         image_rel_path = os.path.splitext(annotation_file.full_path)[0].lstrip("/")
@@ -408,13 +462,15 @@ def export(annotation_files: Iterable[dt.AnnotationFile], output_dir: Path, mode
         height = annotation_file.image_height
         width = annotation_file.image_width
         if height is None or width is None:
-            raise ValueError(f"Annotation file {annotation_file.filename} references an image with no height or width")
+            raise ValueError(
+                f"Annotation file {annotation_file.filename} references an image with no height or width"
+            )
 
         mask: NDArray = np.zeros((height, width)).astype(np.uint8)
         annotations: List[dt.AnnotationLike] = [
             a
             for a in annotation_file.annotations
-            if a.annotation_class.annotation_type in ["polygon", "complex_polygon", "raster_layer", "mask"]
+            if a.annotation_class.annotation_type in accepted_types
         ]
 
         render_type = get_render_mode(annotations)
@@ -439,7 +495,9 @@ def export(annotation_files: Iterable[dt.AnnotationFile], output_dir: Path, mode
             raise DarwinException.from_multiple_exceptions(errors)
 
         # Map to palette
-        mask = np.array(mask, dtype=np.uint8)  # Final double check that type is using correct dtype
+        mask = np.array(
+            mask, dtype=np.uint8
+        )  # Final double check that type is using correct dtype
 
         if mode == "rgb":
             rgb_colours, palette_rgb = get_rgb_colours(categories)
@@ -454,18 +512,98 @@ def export(annotation_files: Iterable[dt.AnnotationFile], output_dir: Path, mode
             image = Image.fromarray(mask)
         image.save(outfile)
 
-    with open(output_dir / "class_mapping.csv", "w") as f:
+    with open(output_dir / "class_mapping.csv", "w", newline="") as f:
         writer = csv_writer(f)
         writer.writerow(["class_name", "class_color"])
 
-        # Create a palette if there was no palette created
-        try:
-            palette_rgb
-        except NameError:
-            palette_rgb = {"__background__": [0, 0, 0]}
-
-        for c in categories:
+        for class_key in categories:
             if mode == "rgb":
-                writer.writerow([c, f"{palette_rgb[c][0]} {palette_rgb[c][1]} {palette_rgb[c][2]}"])
+                col = palette_rgb[class_key]
+                writer.writerow([class_key, f"{col[0]} {col[1]} {col[2]}"])
             else:
-                writer.writerow([c, f"{palette[c]}"])
+                writer.writerow([class_key, f"{palette[class_key]}"])
+
+
+def annotations_exceed_window(
+    annotations: List[dt.Annotation], height: int, width: int
+) -> bool:
+    """Check if any annotations exceed the image window
+
+    Args:
+        annotations (List[dt.Annotation]): List of annotations
+        height (int): height of image
+        width (int): width of image
+
+    Returns:
+        bool: True if any annotation exceeds window, false otherwise
+    """
+    for item in annotations:
+        if "bounding_box" not in item.data:
+            continue
+        bbox = item.data["bounding_box"]
+        if bbox["x"] < 0:
+            return True
+        if bbox["y"] < 0:
+            return True
+        if bbox["x"] + bbox["w"] > width:
+            return True
+        if bbox["y"] + bbox["h"] > height:
+            return True
+    return False
+
+
+def get_extents(
+    annotations: List[dt.Annotation], height: int = 0, width: int = 0
+) -> Tuple[int, int, int, int]:
+    """Create a bounding box around all annotations in discrete pixel space
+
+    Args:
+        annotations (List[dt.Annotation]): List of annotations
+        height (int): Height to start with
+        width (int): Width to start with
+
+    Returns:
+        Tuple[int, int, int, int]: x_min, x_max, y_min, y_max
+    """
+    x_min = y_min = 0
+    x_max, y_max = width, height
+    for item in annotations:
+        if "bounding_box" not in item.data:
+            continue
+        bbox = item.data["bounding_box"]
+        x_min = min(x_min, bbox["x"])
+        x_max = max(x_max, bbox["x"] + bbox["w"])
+        y_min = min(y_min, bbox["y"])
+        y_max = max(y_max, bbox["y"] + bbox["h"])
+    return math.floor(x_min), math.ceil(x_max), math.floor(y_min), math.ceil(y_max)
+
+
+def offset_polygon(polygon: List, offset_x: int, offset_y: int) -> List:
+    """Offsets a polygon by a given amount
+
+    Args:
+        polygon (List): List of coordinates
+        offset_x (int): x offset value
+        offset_y (int): y offset value
+
+    Returns:
+        List: polygon with offset applied
+    """
+    if isinstance(polygon[0], list):
+        return offset_complex_polygon(polygon, offset_x, offset_y)
+    else:
+        return offset_simple_polygon(polygon, offset_x, offset_y)
+
+
+def offset_complex_polygon(polygons: List, offset_x: int, offset_y: int) -> List:
+    new_polygons = []
+    for polygon in polygons:
+        new_polygons.append(offset_simple_polygon(polygon, offset_x, offset_y))
+    return new_polygons
+
+
+def offset_simple_polygon(polygon: List, offset_x: int, offset_y: int) -> List:
+    new_polygon = []
+    for point in polygon:
+        new_polygon.append({"x": point["x"] + offset_x, "y": point["y"] + offset_y})
+    return new_polygon
