@@ -1,10 +1,8 @@
-from pydantic import parse_obj_as
-
-from darwin.future.core.client import Client
-from darwin.future.data_objects.dataset import Dataset
+from darwin.future.core.client import ClientCore
+from darwin.future.data_objects.dataset import DatasetCore
 
 
-def create_dataset(api_client: Client, name: str) -> Dataset:
+def create_dataset(api_client: ClientCore, name: str) -> DatasetCore:
     """
     Creates a new dataset for the given team
 
@@ -32,5 +30,5 @@ def create_dataset(api_client: Client, name: str) -> Dataset:
             "name": name,
         },
     )
-
-    return parse_obj_as(Dataset, response)
+    assert isinstance(response, dict)
+    return DatasetCore.model_validate(response)
