@@ -1,11 +1,9 @@
-from pydantic import parse_obj_as
-
-from darwin.future.core.client import Client
+from darwin.future.core.client import ClientCore
 from darwin.future.core.types.common import QueryString
-from darwin.future.data_objects.dataset import Dataset
+from darwin.future.data_objects.dataset import DatasetCore
 
 
-def get_dataset(api_client: Client, dataset_id: str) -> Dataset:
+def get_dataset(api_client: ClientCore, dataset_id: str) -> DatasetCore:
     """
     Returns a list of datasets for the given team
 
@@ -29,5 +27,5 @@ def get_dataset(api_client: Client, dataset_id: str) -> Dataset:
     """
 
     response = api_client.get("/datasets", QueryString({"id": str(dataset_id)}))
-
-    return parse_obj_as(Dataset, response)
+    assert isinstance(response, dict)
+    return DatasetCore.model_validate(response)
