@@ -7,8 +7,10 @@ from darwin.future.data_objects.validators import parse_name
 from darwin.future.pydantic_base import DefaultDarwin
 
 
-class Dataset(DefaultDarwin):
-    """A class to manage all the information around a dataset on the darwin platform, including validation
+class DatasetCore(DefaultDarwin):
+    """
+    A class to manage all the information around a dataset on the darwin platform,
+    including validation
 
     Attributes
     ----------
@@ -31,4 +33,4 @@ class Dataset(DefaultDarwin):
     _name_validator = validator("name", allow_reuse=True)(parse_name)
 
 
-DatasetList = List[Dataset]
+DatasetList = List[DatasetCore]
