@@ -6,8 +6,11 @@ from darwin.future.data_objects import validators as darwin_validators
 from darwin.future.pydantic_base import DefaultDarwin
 
 
-class Release(DefaultDarwin):
-    """A class to manage all the information around a release on the darwin platform, including validation
+class ReleaseCore(DefaultDarwin):
+    """
+    A class to manage all the information around a release on the darwin platform
+    including validation
+
     Attributes
     ----------
     name : str
@@ -19,8 +22,11 @@ class Release(DefaultDarwin):
 
     name: str
 
+    def __str__(self) -> str:
+        return self.name
+
     # Data Validation
     _name_validator = validator("name", allow_reuse=True)(darwin_validators.parse_name)
 
 
-ReleaseList = List[Release]
+ReleaseList = List[ReleaseCore]
