@@ -15,7 +15,7 @@ try:
     import torch  # noqa
 except ImportError:
     raise ImportError(
-        f"darwin.torch requires pytorch and torchvision. Install it using: pip install torch torchvision"
+        "darwin.torch requires pytorch and torchvision. Install it using: pip install torch torchvision"
     ) from None
 
 from .dataset import get_dataset  # noqa
