@@ -20,24 +20,25 @@ from typing import (
 )
 
 import deprecation
+import json_stream
 import numpy as np
 import orjson as json
 import requests
-from jsonschema import exceptions, validators
-from requests import Response, request
+from json_stream.base import PersistentStreamingJSONList, PersistentStreamingJSONObject
+from jsonschema import validators
+from requests import Response
 from rich.progress import ProgressType, track
 from upolygon import draw_polygon
 
 import darwin.datatypes as dt
 from darwin.config import Config
 from darwin.exceptions import (
-    AnnotationFileValidationError,
     MissingSchema,
     OutdatedDarwinJSONFormat,
-    UnknownAnnotationFileSchema,
     UnrecognizableFileEncoding,
     UnsupportedFileType,
 )
+from darwin.future.data_objects.properties import SelectedProperty
 from darwin.version import __version__
 
 if TYPE_CHECKING:
@@ -87,7 +88,7 @@ def is_extension_allowed_by_filename(filename: str) -> bool:
     bool
         Whether or not the given extension of the filename is allowed.
     """
-    return any([filename.lower().endswith(ext) for ext in SUPPORTED_EXTENSIONS])
+    return any(filename.lower().endswith(ext) for ext in SUPPORTED_EXTENSIONS)
 
 
 @deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
@@ -124,7 +125,7 @@ def is_image_extension_allowed_by_filename(filename: str) -> bool:
     bool
         Whether or not the given extension is allowed.
     """
-    return any([filename.lower().endswith(ext) for ext in SUPPORTED_IMAGE_EXTENSIONS])
+    return any(filename.lower().endswith(ext) for ext in SUPPORTED_IMAGE_EXTENSIONS)
 
 
 @deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
@@ -159,7 +160,7 @@ def is_video_extension_allowed_by_filename(extension: str) -> bool:
     bool
         Whether or not the given extension is allowed.
     """
-    return any([extension.lower().endswith(ext) for ext in SUPPORTED_VIDEO_EXTENSIONS])
+    return any(extension.lower().endswith(ext) for ext in SUPPORTED_VIDEO_EXTENSIONS)
 
 
 @deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
@@ -214,7 +215,9 @@ def is_project_dir(project_path: Path) -> bool:
     return (project_path / "releases").exists() and (project_path / "images").exists()
 
 
-def get_progress_bar(array: List[dt.AnnotationFile], description: Optional[str] = None) -> Iterable[ProgressType]:
+def get_progress_bar(
+    array: List[dt.AnnotationFile], description: Optional[str] = None
+) -> Iterable[ProgressType]:
     """
     Get a rich a progress bar for the given list of annotation files.
 
@@ -263,7 +266,10 @@ def prompt(msg: str, default: Optional[str] = None) -> str:
 
 
 def find_files(
-    files: List[dt.PathLike], *, files_to_exclude: List[dt.PathLike] = [], recursive: bool = True
+    files: List[dt.PathLike],
+    *,
+    files_to_exclude: List[dt.PathLike] = [],
+    recursive: bool = True,
 ) -> List[Path]:
     """
     Retrieve a list of all files belonging to supported extensions. The exploration can be made
@@ -289,7 +295,7 @@ def find_files(
 
     for f in files:
         path = Path(f)
-        if path.is_dir() == True:
+        if path.is_dir():
             found_files.extend(
                 [
                     path_object
@@ -320,7 +326,9 @@ def secure_continue_request() -> bool:
 
 
 def persist_client_configuration(
-    client: "Client", default_team: Optional[str] = None, config_path: Optional[Path] = None
+    client: "Client",
+    default_team: Optional[str] = None,
+    config_path: Optional[Path] = None,
 ) -> Config:
     """
     Authenticate user against the server and creates a configuration file for him/her.
@@ -348,8 +356,14 @@ def persist_client_configuration(
         raise ValueError("Unable to get default team.")
 
     config: Config = Config(config_path)
-    config.set_team(team=team_config.slug, api_key=team_config.api_key, datasets_dir=team_config.datasets_dir)
-    config.set_global(api_endpoint=client.url, base_url=client.base_url, default_team=default_team)
+    config.set_team(
+        team=team_config.slug,
+        api_key=team_config.api_key,
+        datasets_dir=team_config.datasets_dir,
+    )
+    config.set_global(
+        api_endpoint=client.url, base_url=client.base_url, default_team=default_team
+    )
 
     return config
 
@@ -406,7 +420,9 @@ def attempt_decode(path: Path) -> dict:
             return data
         except Exception:
             continue
-    raise UnrecognizableFileEncoding(f"Unable to load file {path} with any encodings: {encodings}")
+    raise UnrecognizableFileEncoding(
+        f"Unable to load file {path} with any encodings: {encodings}"
+    )
 
 
 def load_data_from_file(path: Path) -> Tuple[dict, dt.AnnotationFileVersion]:
@@ -415,7 +431,9 @@ def load_data_from_file(path: Path) -> Tuple[dict, dt.AnnotationFileVersion]:
     return data, version
 
 
-def parse_darwin_json(path: Path, count: Optional[int]) -> Optional[dt.AnnotationFile]:
+def parse_darwin_json(
+    path: Path, count: Optional[int] = None
+) -> Optional[dt.AnnotationFile]:
     """
     Parses the given JSON file in v7's darwin proprietary format. Works for images, split frame
     videos (treated as images) and playback videos.
@@ -455,12 +473,122 @@ def parse_darwin_json(path: Path, count: Optional[int]) -> Optional[dt.Annotatio
             return _parse_darwin_image(path, data, count)
 
 
+def stream_darwin_json(path: Path) -> PersistentStreamingJSONObject:
+    """
+    Returns a Darwin JSON file as a persistent stream. This allows for parsing large files without
+    loading them entirely into memory.
+
+    Parameters
+    ----------
+    path : Path
+        Path to the file to parse.
+
+    Returns
+    -------
+    PersistentStreamingJSONObject
+        A stream of the JSON file.
+    """
+
+    with path.open() as infile:
+        return json_stream.load(infile, persistent=True)
+
+
+def get_image_path_from_stream(
+    darwin_json: PersistentStreamingJSONObject,
+    images_dir: Path,
+    with_folders: bool,
+    json_version: str,
+    annotation_filepath: Path,
+) -> Path:
+    """
+    Returns the path to the image file associated with the given darwin json file.
+    Compatible with V1 & V2 Darwin JSON, as well as releases in folders and flat structures.
+
+    Parameters
+    ----------
+    darwin_json : PersistentStreamingJSONObject
+        A stream of the JSON file.
+    images_dir : Path
+        Path to the directory containing the images.
+    with_folders: bool
+        Flag to determine if the release was pulled with or without folders.
+    json_version: str
+        String representing the version of the Darwin JSON
+
+    Returns
+    -------
+    Path
+        Path to the image file.
+    """
+    try:
+        if json_version == "2.0":
+            if not with_folders:
+                return images_dir / Path(darwin_json["item"]["name"])
+            else:
+                return (
+                    images_dir
+                    / (Path(darwin_json["item"]["path"].lstrip("/\\")))
+                    / Path(darwin_json["item"]["name"])
+                )
+        else:
+            if not with_folders:
+                return images_dir / Path(darwin_json["image"]["filename"])
+            else:
+                return (
+                    images_dir
+                    / (Path(darwin_json["image"]["path"].lstrip("/\\")))
+                    / Path(darwin_json["image"]["filename"])
+                )
+    except OSError as e:
+        # Load in the JSON as normal
+        darwin_json = parse_darwin_json(path=annotation_filepath)
+        if not with_folders:
+            return images_dir / Path(darwin_json.filename)
+        else:
+            return images_dir / Path(darwin_json.full_path.lstrip("/\\"))
+
+
+def get_darwin_json_version(annotations_dir: Path) -> str:
+    """
+    Returns true is the input Darwin JSON file is 2.0, and False if 1.0.
+
+    Parameters
+    ----------
+    annotations_dir : Path
+        Path to the directory containing the annotation files.
+
+    Returns
+    -------
+    str
+        A str representing the Darwin JSON version.
+    """
+    with open(next(annotations_dir.glob("*.json")), "r") as file:
+        data_str = file.read()
+        data = json.loads(data_str)
+        return "2.0" if "version" in data and data["version"] == "2.0" else "1.0"
+
+
+def is_stream_list_empty(json_list: PersistentStreamingJSONList) -> bool:
+    try:
+        json_list[0]
+    except IndexError:
+        return True
+
+    return False
+
+
 def _parse_darwin_v2(path: Path, data: Dict[str, Any]) -> dt.AnnotationFile:
     item = data["item"]
     item_source = item.get("source_info", {})
-    slots: List[dt.Slot] = list(filter(None, map(_parse_darwin_slot, item.get("slots", []))))
-    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(data)
-    annotation_classes: Set[dt.AnnotationClass] = set([annotation.annotation_class for annotation in annotations])
+    slots: List[dt.Slot] = list(
+        filter(None, map(_parse_darwin_slot, item.get("slots", [])))
+    )
+    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(
+        data
+    )
+    annotation_classes: Set[dt.AnnotationClass] = {
+        annotation.annotation_class for annotation in annotations
+    }
 
     if len(slots) == 0:
         annotation_file = dt.AnnotationFile(
@@ -468,7 +596,9 @@ def _parse_darwin_v2(path: Path, data: Dict[str, Any]) -> dt.AnnotationFile:
             path=path,
             filename=item["name"],
             item_id=item.get("source_info", {}).get("item_id", None),
-            dataset_name=item.get("source_info", {}).get("dataset", {}).get("name", None),
+            dataset_name=item.get("source_info", {})
+            .get("dataset", {})
+            .get("name", None),
             annotation_classes=annotation_classes,
             annotations=annotations,
             is_video=False,
@@ -489,19 +619,24 @@ def _parse_darwin_v2(path: Path, data: Dict[str, Any]) -> dt.AnnotationFile:
             path=path,
             filename=item["name"],
             item_id=item.get("source_info", {}).get("item_id", None),
-            dataset_name=item.get("source_info", {}).get("dataset", {}).get("name", None),
+            dataset_name=item.get("source_info", {})
+            .get("dataset", {})
+            .get("name", None),
             annotation_classes=annotation_classes,
             annotations=annotations,
-            is_video=slot.frame_urls is not None,
+            is_video=slot.frame_urls is not None or slot.frame_manifest is not None,
             image_width=slot.width,
             image_height=slot.height,
-            image_url=None if len(slot.source_files or []) == 0 else slot.source_files[0]["url"],
+            image_url=None
+            if len(slot.source_files or []) == 0
+            else slot.source_files[0]["url"],
             image_thumbnail_url=slot.thumbnail_url,
-            workview_url=item_source["workview_url"],
+            workview_url=item_source.get("workview_url", None),
             seq=0,
             frame_urls=slot.frame_urls,
             remote_path=item["path"],
             slots=slots,
+            frame_count=slot.frame_count,
         )
 
     return annotation_file
@@ -519,17 +654,30 @@ def _parse_darwin_slot(data: Dict[str, Any]) -> dt.Slot:
         frame_urls=data.get("frame_urls"),
         fps=data.get("fps"),
         metadata=data.get("metadata"),
+        segments=data.get("segments", []),
+        frame_manifest=data.get("frame_manifests"),
     )
 
 
-def _parse_darwin_image(path: Path, data: Dict[str, Any], count: Optional[int]) -> dt.AnnotationFile:
-    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(data)
-    annotation_classes: Set[dt.AnnotationClass] = set([annotation.annotation_class for annotation in annotations])
+def _parse_darwin_image(
+    path: Path, data: Dict[str, Any], count: Optional[int]
+) -> dt.AnnotationFile:
+    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(
+        data
+    )
+    annotation_classes: Set[dt.AnnotationClass] = {
+        annotation.annotation_class for annotation in annotations
+    }
 
     slot = dt.Slot(
         name=None,
         type="image",
-        source_files=[{"url": data["image"].get("url"), "file_name": _get_local_filename(data["image"])}],
+        source_files=[
+            {
+                "url": data["image"].get("url"),
+                "file_name": _get_local_filename(data["image"]),
+            }
+        ],
         thumbnail_url=data["image"].get("thumbnail_url"),
         width=data["image"].get("width"),
         height=data["image"].get("height"),
@@ -556,17 +704,30 @@ def _parse_darwin_image(path: Path, data: Dict[str, Any], count: Optional[int])
     return annotation_file
 
 
-def _parse_darwin_video(path: Path, data: Dict[str, Any], count: Optional[int]) -> dt.AnnotationFile:
-    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(data)
-    annotation_classes: Set[dt.AnnotationClass] = set([annotation.annotation_class for annotation in annotations])
+def _parse_darwin_video(
+    path: Path, data: Dict[str, Any], count: Optional[int]
+) -> dt.AnnotationFile:
+    annotations: List[Union[dt.Annotation, dt.VideoAnnotation]] = _data_to_annotations(
+        data
+    )
+    annotation_classes: Set[dt.AnnotationClass] = {
+        annotation.annotation_class for annotation in annotations
+    }
 
     if "width" not in data["image"] or "height" not in data["image"]:
-        raise OutdatedDarwinJSONFormat("Missing width/height in video, please re-export")
+        raise OutdatedDarwinJSONFormat(
+            "Missing width/height in video, please re-export"
+        )
 
     slot = dt.Slot(
         name=None,
         type="video",
-        source_files=[{"url": data["image"].get("url"), "file_name": _get_local_filename(data["image"])}],
+        source_files=[
+            {
+                "url": data["image"].get("url"),
+                "file_name": _get_local_filename(data["image"]),
+            }
+        ],
         thumbnail_url=data["image"].get("thumbnail_url"),
         width=data["image"].get("width"),
         height=data["image"].get("height"),
@@ -602,23 +763,41 @@ def _parse_darwin_annotation(annotation: Dict[str, Any]) -> Optional[dt.Annotati
     main_annotation: Optional[dt.Annotation] = None
 
     # Darwin JSON 2.0 representation of complex polygons
-    if "polygon" in annotation and "paths" in annotation["polygon"] and len(annotation["polygon"]["paths"]) > 1:
+    if (
+        "polygon" in annotation
+        and "paths" in annotation["polygon"]
+        and len(annotation["polygon"]["paths"]) > 1
+    ):
         bounding_box = annotation.get("bounding_box")
         paths = annotation["polygon"]["paths"]
-        main_annotation = dt.make_complex_polygon(name, paths, bounding_box, slot_names=slot_names)
+        main_annotation = dt.make_complex_polygon(
+            name, paths, bounding_box, slot_names=slot_names
+        )
     # Darwin JSON 2.0 representation of simple polygons
-    elif "polygon" in annotation and "paths" in annotation["polygon"] and len(annotation["polygon"]["paths"]) == 1:
+    elif (
+        "polygon" in annotation
+        and "paths" in annotation["polygon"]
+        and len(annotation["polygon"]["paths"]) == 1
+    ):
         bounding_box = annotation.get("bounding_box")
         paths = annotation["polygon"]["paths"]
-        main_annotation = dt.make_polygon(name, paths[0], bounding_box, slot_names=slot_names)
+        main_annotation = dt.make_polygon(
+            name, paths[0], bounding_box, slot_names=slot_names
+        )
     # Darwin JSON 1.0 representation of complex and simple polygons
     elif "polygon" in annotation:
         bounding_box = annotation.get("bounding_box")
         if "additional_paths" in annotation["polygon"]:
-            paths = [annotation["polygon"]["path"]] + annotation["polygon"]["additional_paths"]
-            main_annotation = dt.make_complex_polygon(name, paths, bounding_box, slot_names=slot_names)
+            paths = [annotation["polygon"]["path"]] + annotation["polygon"][
+                "additional_paths"
+            ]
+            main_annotation = dt.make_complex_polygon(
+                name, paths, bounding_box, slot_names=slot_names
+            )
         else:
-            main_annotation = dt.make_polygon(name, annotation["polygon"]["path"], bounding_box, slot_names=slot_names)
+            main_annotation = dt.make_polygon(
+                name, annotation["polygon"]["path"], bounding_box, slot_names=slot_names
+            )
     # Darwin JSON 1.0 representation of complex polygons
     elif "complex_polygon" in annotation:
         bounding_box = annotation.get("bounding_box")
@@ -630,35 +809,72 @@ def _parse_darwin_annotation(annotation: Dict[str, Any]) -> Optional[dt.Annotati
         if "additional_paths" in annotation["complex_polygon"]:
             paths.extend(annotation["complex_polygon"]["additional_paths"])
 
-        main_annotation = dt.make_complex_polygon(name, paths, bounding_box, slot_names=slot_names)
+        main_annotation = dt.make_complex_polygon(
+            name, paths, bounding_box, slot_names=slot_names
+        )
     elif "bounding_box" in annotation:
         bounding_box = annotation["bounding_box"]
         main_annotation = dt.make_bounding_box(
-            name, bounding_box["x"], bounding_box["y"], bounding_box["w"], bounding_box["h"], slot_names=slot_names
+            name,
+            bounding_box["x"],
+            bounding_box["y"],
+            bounding_box["w"],
+            bounding_box["h"],
+            slot_names=slot_names,
         )
     elif "tag" in annotation:
         main_annotation = dt.make_tag(name, slot_names=slot_names)
     elif "line" in annotation:
-        main_annotation = dt.make_line(name, annotation["line"]["path"], slot_names=slot_names)
+        main_annotation = dt.make_line(
+            name, annotation["line"]["path"], slot_names=slot_names
+        )
     elif "keypoint" in annotation:
         main_annotation = dt.make_keypoint(
-            name, annotation["keypoint"]["x"], annotation["keypoint"]["y"], slot_names=slot_names
+            name,
+            annotation["keypoint"]["x"],
+            annotation["keypoint"]["y"],
+            slot_names=slot_names,
         )
     elif "ellipse" in annotation:
-        main_annotation = dt.make_ellipse(name, annotation["ellipse"], slot_names=slot_names)
+        main_annotation = dt.make_ellipse(
+            name, annotation["ellipse"], slot_names=slot_names
+        )
     elif "cuboid" in annotation:
-        main_annotation = dt.make_cuboid(name, annotation["cuboid"], slot_names=slot_names)
+        main_annotation = dt.make_cuboid(
+            name, annotation["cuboid"], slot_names=slot_names
+        )
     elif "skeleton" in annotation:
-        main_annotation = dt.make_skeleton(name, annotation["skeleton"]["nodes"], slot_names=slot_names)
+        main_annotation = dt.make_skeleton(
+            name, annotation["skeleton"]["nodes"], slot_names=slot_names
+        )
     elif "table" in annotation:
         main_annotation = dt.make_table(
-            name, annotation["table"]["bounding_box"], annotation["table"]["cells"], slot_names=slot_names
+            name,
+            annotation["table"]["bounding_box"],
+            annotation["table"]["cells"],
+            slot_names=slot_names,
         )
     elif "string" in annotation:
-        main_annotation = dt.make_string(name, annotation["string"]["sources"], slot_names=slot_names)
+        main_annotation = dt.make_string(
+            name, annotation["string"]["sources"], slot_names=slot_names
+        )
     elif "graph" in annotation:
         main_annotation = dt.make_graph(
-            name, annotation["graph"]["nodes"], annotation["graph"]["edges"], slot_names=slot_names
+            name,
+            annotation["graph"]["nodes"],
+            annotation["graph"]["edges"],
+            slot_names=slot_names,
+        )
+    elif "mask" in annotation:
+        main_annotation = dt.make_mask(name, slot_names=slot_names)
+    elif "raster_layer" in annotation:
+        raster_layer = annotation["raster_layer"]
+        main_annotation = dt.make_raster_layer(
+            name,
+            raster_layer["mask_annotation_ids_mapping"],
+            raster_layer["total_pixels"],
+            raster_layer["dense_rle"],
+            slot_names=slot_names,
         )
 
     if not main_annotation:
@@ -668,19 +884,29 @@ def _parse_darwin_annotation(annotation: Dict[str, Any]) -> Optional[dt.Annotati
     if "id" in annotation:
         main_annotation.id = annotation["id"]
     if "instance_id" in annotation:
-        main_annotation.subs.append(dt.make_instance_id(annotation["instance_id"]["value"]))
+        main_annotation.subs.append(
+            dt.make_instance_id(annotation["instance_id"]["value"])
+        )
     if "attributes" in annotation:
         main_annotation.subs.append(dt.make_attributes(annotation["attributes"]))
     if "text" in annotation:
         main_annotation.subs.append(dt.make_text(annotation["text"]["text"]))
     if "inference" in annotation:
-        main_annotation.subs.append(dt.make_opaque_sub("inference", annotation["inference"]))
+        main_annotation.subs.append(
+            dt.make_opaque_sub("inference", annotation["inference"])
+        )
     if "directional_vector" in annotation:
-        main_annotation.subs.append(dt.make_opaque_sub("directional_vector", annotation["directional_vector"]))
+        main_annotation.subs.append(
+            dt.make_opaque_sub("directional_vector", annotation["directional_vector"])
+        )
     if "measures" in annotation:
-        main_annotation.subs.append(dt.make_opaque_sub("measures", annotation["measures"]))
+        main_annotation.subs.append(
+            dt.make_opaque_sub("measures", annotation["measures"])
+        )
     if "auto_annotate" in annotation:
-        main_annotation.subs.append(dt.make_opaque_sub("auto_annotate", annotation["auto_annotate"]))
+        main_annotation.subs.append(
+            dt.make_opaque_sub("auto_annotate", annotation["auto_annotate"])
+        )
 
     if annotation.get("annotators") is not None:
         main_annotation.annotators = _parse_annotators(annotation["annotators"])
@@ -688,6 +914,9 @@ def _parse_darwin_annotation(annotation: Dict[str, Any]) -> Optional[dt.Annotati
     if annotation.get("reviewers") is not None:
         main_annotation.reviewers = _parse_annotators(annotation["reviewers"])
 
+    if "properties" in annotation:
+        main_annotation.properties = _parse_properties(annotation["properties"])
+
     return main_annotation
 
 
@@ -710,6 +939,7 @@ def _parse_darwin_video_annotation(annotation: dict) -> Optional[dt.VideoAnnotat
         annotation.get("ranges", annotation.get("segments", [])),
         annotation.get("interpolated", False),
         slot_names=parse_slot_names(annotation),
+        properties=_parse_properties(annotation.get("properties", [])),
     )
 
     if "id" in annotation:
@@ -733,8 +963,10 @@ def _parse_darwin_raster_annotation(annotation: dict) -> Optional[dt.Annotation]
     raster_layer: Optional[dt.JSONFreeForm] = annotation.get("raster_layer")
     slot_names: Optional[List[str]] = parse_slot_names(annotation)
 
-    if not id or not name or not raster_layer or not slot_names:
-        raise ValueError("Raster annotation must have an 'id', 'name', 'slot_names' and 'raster_layer' field")
+    if not id or not name or not raster_layer:
+        raise ValueError(
+            "Raster annotation must have an 'id', 'name' and 'raster_layer' field"
+        )
 
     dense_rle, mask_annotation_ids_mapping, total_pixels = (
         raster_layer.get("dense_rle", None),
@@ -750,43 +982,34 @@ def _parse_darwin_raster_annotation(annotation: dict) -> Optional[dt.Annotation]
     new_annotation = dt.Annotation(
         dt.AnnotationClass(name, "raster_layer"),
         {
-            "id": id,
-            "name": name,
-            "raster_layer": {
-                "dense_rle": dense_rle,
-                "mask_annotation_ids_mapping": mask_annotation_ids_mapping,
-                "total_pixels": total_pixels,
-            },
+            "dense_rle": dense_rle,
+            "mask_annotation_ids_mapping": mask_annotation_ids_mapping,
+            "total_pixels": total_pixels,
         },
-        slot_names=slot_names,
+        slot_names=slot_names or [],
+        id=id,
     )
 
     return new_annotation
 
 
 def _parse_darwin_mask_annotation(annotation: dict) -> Optional[dt.Annotation]:
-    if not annotation.get("mask"):
-        raise ValueError("Mask annotation must have a 'mask' field")
-
     id: Optional[str] = annotation.get("id")
     name: Optional[str] = annotation.get("name")
     mask: Optional[dt.JSONFreeForm] = annotation.get("mask")
     slot_names: Optional[List[str]] = parse_slot_names(annotation)
 
-    if not id or not name or not mask or not slot_names:
-        raise ValueError("Mask annotation must have an 'id', 'name', 'slot_names' and 'mask' field")
+    if not id or not name or mask is None:
+        raise ValueError("Mask annotation must have an 'id', 'name' and 'mask' field")
 
-    if (not "sparse_rle" in mask) or (not mask["sparse_rle"]) == None:
-        raise ValueError("Mask annotation must have a 'sparse_rle' field containing a null value")
+    if ("sparse_rle" in mask) and (mask["sparse_rle"] is not None):
+        raise ValueError("Mask annotation field 'sparse_rle' must contain a null value")
 
     new_annotation = dt.Annotation(
         dt.AnnotationClass(name, "mask"),
-        {
-            "id": id,
-            "name": name,
-            "mask": mask,
-        },
-        slot_names=slot_names,
+        mask,
+        slot_names=slot_names or [],
+        id=id,
     )
 
     return new_annotation
@@ -794,9 +1017,30 @@ def _parse_darwin_mask_annotation(annotation: dict) -> Optional[dt.Annotation]:
 
 def _parse_annotators(annotators: List[Dict[str, Any]]) -> List[dt.AnnotationAuthor]:
     if not (hasattr(annotators, "full_name") or not hasattr(annotators, "email")):
-        raise AttributeError("JSON file must contain annotators with 'full_name' and 'email' fields")
+        raise AttributeError(
+            "JSON file must contain annotators with 'full_name' and 'email' fields"
+        )
 
-    return [dt.AnnotationAuthor(annotator["full_name"], annotator["email"]) for annotator in annotators]
+    return [
+        dt.AnnotationAuthor(annotator["full_name"], annotator["email"])
+        for annotator in annotators
+    ]
+
+
+def _parse_properties(
+    properties: List[Dict[str, Any]]
+) -> Optional[List[SelectedProperty]]:
+    selected_properties = []
+    for property in properties:
+        selected_properties.append(
+            SelectedProperty(
+                frame_index=property.get("frame_index", None),
+                name=property.get("name", None),
+                value=property.get("value", None),
+            )
+        )
+
+    return selected_properties or None
 
 
 def split_video_annotation(annotation: dt.AnnotationFile) -> List[dt.AnnotationFile]:
@@ -821,17 +1065,24 @@ def split_video_annotation(annotation: dt.AnnotationFile) -> List[dt.AnnotationF
         no ``frame_url`` attribute.
     """
     if not annotation.is_video:
-        raise AttributeError("this is not a video annotation")
-
-    if not annotation.frame_urls:
-        raise AttributeError("This Annotation has no frame urls")
+        raise AttributeError("This is not a video annotation")
 
+    # changes here from annotation.frame_urls to annotation.frame_count with frame_urls as backup
+    # due to addition of long videos feature, where frame_urls is no longer available.
+    # frame_count should be available for both, however existing annotations will not have this
+    if not annotation.frame_count and not annotation.frame_urls:
+        raise AttributeError("This Annotation has no frames")
+    urls = annotation.frame_urls or [None] * (annotation.frame_count or 1)
     frame_annotations = []
-    for i, frame_url in enumerate(annotation.frame_urls):
+    for i, frame_url in enumerate(urls):
         annotations = [
-            a.frames[i] for a in annotation.annotations if isinstance(a, dt.VideoAnnotation) and i in a.frames
+            a.frames[i]
+            for a in annotation.annotations
+            if isinstance(a, dt.VideoAnnotation) and i in a.frames
         ]
-        annotation_classes: Set[dt.AnnotationClass] = set([annotation.annotation_class for annotation in annotations])
+        annotation_classes: Set[dt.AnnotationClass] = {
+            annotation.annotation_class for annotation in annotations
+        }
         filename: str = f"{Path(annotation.filename).stem}/{i:07d}.png"
         frame_annotations.append(
             dt.AnnotationFile(
@@ -847,8 +1098,10 @@ def split_video_annotation(annotation: dt.AnnotationFile) -> List[dt.AnnotationF
                 annotation.seq,
                 item_id=annotation.item_id,
                 slots=annotation.slots,
+                remote_path=annotation.remote_path,
             )
         )
+
     return frame_annotations
 
 
@@ -915,7 +1168,9 @@ def convert_polygons_to_sequences(
     else:
         list_polygons = cast(List[dt.Polygon], [polygons])
 
-    if not isinstance(list_polygons[0], list) or not isinstance(list_polygons[0][0], dict):
+    if not isinstance(list_polygons[0], list) or not isinstance(
+        list_polygons[0][0], dict
+    ):
         raise ValueError("Unknown input format")
 
     sequences: List[List[Union[int, float]]] = []
@@ -942,7 +1197,9 @@ def convert_polygons_to_sequences(
     details="Do not use.",
 )
 def convert_sequences_to_polygons(
-    sequences: List[Union[List[int], List[float]]], height: Optional[int] = None, width: Optional[int] = None
+    sequences: List[Union[List[int], List[float]]],
+    height: Optional[int] = None,
+    width: Optional[int] = None,
 ) -> Dict[str, List[dt.Polygon]]:
     """
     Converts a list of polygons, encoded as a list of dictionaries of into a list of nd.arrays
@@ -1054,7 +1311,9 @@ def convert_bounding_box_to_xyxy(box: dt.BoundingBox) -> List[float]:
     return [box["x"], box["y"], x2, y2]
 
 
-def convert_polygons_to_mask(polygons: List, height: int, width: int, value: Optional[int] = 1) -> np.ndarray:
+def convert_polygons_to_mask(
+    polygons: List, height: int, width: int, value: Optional[int] = 1
+) -> np.ndarray:
     """
     Converts a list of polygons, encoded as a list of dictionaries into an ``nd.array`` mask.
 
@@ -1148,31 +1407,58 @@ def _parse_version(data: dict) -> dt.AnnotationFileVersion:
     return dt.AnnotationFileVersion(int(major), int(minor), suffix)
 
 
-def _data_to_annotations(data: Dict[str, Any]) -> List[Union[dt.Annotation, dt.VideoAnnotation]]:
+def _data_to_annotations(
+    data: Dict[str, Any]
+) -> List[Union[dt.Annotation, dt.VideoAnnotation]]:
     raw_image_annotations = filter(
         lambda annotation: (
-            ("frames" not in annotation) and ("raster_layer" not in annotation) and ("mask" not in annotation)
+            ("frames" not in annotation)
+            and ("raster_layer" not in annotation)
+            and ("mask" not in annotation)
         ),
         data["annotations"],
     )
-    raw_video_annotations = filter(lambda annotation: "frames" in annotation, data["annotations"])
-    raw_raster_annotations = filter(lambda annotation: "raster_layer" in annotation, data["annotations"])
-    raw_mask_annotations = filter(lambda annotation: "mask" in annotation, data["annotations"])
-    image_annotations: List[dt.Annotation] = list(filter(None, map(_parse_darwin_annotation, raw_image_annotations)))
+    raw_video_annotations = filter(
+        lambda annotation: "frames" in annotation, data["annotations"]
+    )
+    raw_raster_annotations = filter(
+        lambda annotation: "raster_layer" in annotation, data["annotations"]
+    )
+    raw_mask_annotations = filter(
+        lambda annotation: "mask" in annotation, data["annotations"]
+    )
+    image_annotations: List[dt.Annotation] = list(
+        filter(None, map(_parse_darwin_annotation, raw_image_annotations))
+    )
     video_annotations: List[dt.VideoAnnotation] = list(
         filter(None, map(_parse_darwin_video_annotation, raw_video_annotations))
     )
     raster_annotations: List[dt.Annotation] = list(
         filter(None, map(_parse_darwin_raster_annotation, raw_raster_annotations))
     )
-    mask_annotations: List[dt.Annotation] = list(filter(None, map(_parse_darwin_mask_annotation, raw_mask_annotations)))
+    mask_annotations: List[dt.Annotation] = list(
+        filter(None, map(_parse_darwin_mask_annotation, raw_mask_annotations))
+    )
 
-    return [*image_annotations, *video_annotations, *raster_annotations, *mask_annotations]
+    return [
+        *image_annotations,
+        *video_annotations,
+        *raster_annotations,
+        *mask_annotations,
+    ]
 
 
 def _supported_schema_versions() -> Dict[Tuple[int, int, str], str]:
-    return {(2, 0, ""): "https://darwin-public.s3.eu-west-1.amazonaws.com/darwin_json/2.0/schema.json"}
+    return {
+        (
+            2,
+            0,
+            "",
+        ): "https://darwin-public.s3.eu-west-1.amazonaws.com/darwin_json/2.0/schema.json"
+    }
 
 
 def _default_schema(version: dt.AnnotationFileVersion) -> Optional[str]:
-    return _supported_schema_versions().get((version.major, version.minor, version.suffix))
+    return _supported_schema_versions().get(
+        (version.major, version.minor, version.suffix)
+    )
